import click
from .relat import Relat
import yaml


@click.group()
def relaty():
    pass


@click.command(help="Play a story")
@click.argument("story_path")
def play(story_path):
    document=yaml.safe_load(open(story_path))
    relat = Relat.create_from_document(document=document)

    relat.play()


relaty.add_command(play)

if __name__ == "__main__":
    relaty()



import unittest
from yaml import safe_load

from relaty.relat import Relat
from relaty.story import Story


class TestRelaty(unittest.TestCase):

    def setUp(self):
        super()

        document = """
        title: A title
        screens:
            - Hello, how are you?
            - These are two screens
        options:
            - title: option 1
              screens:
                - option 1 screen 1
                - option 1 screen 2
              options:
                - title: option 1.1
                  screens:
                    - option 1.1 screen 1
                    - option 1.1 screen 2
                    - FIN
                - title: option 1.2
                  screens:
                    - option 1.2 screen 1
                    - option 1.2 screen 2
                  options:
                    - title: option 1.2.1
                      screens:
                        - option 1.2.1 screen 1
                        - option 1.2.1 screen 2
                        - FIN
                    - title: option 1.2.2
                      screens:
                        - option 1.2.2 screen 1
                        - option 1.2.2 screen 2
                        - FIN
            - title: Option 2
              screens:
                - option 2 screen 1
                - option 2 screen 2
                - FIN
        """

        converted_document = safe_load(document)
        self.relat_from_document = Relat.create_from_document(
            converted_document)
        self.story_from_document = self.relat_from_document.story

        relat_title = "A test Relat"
        self.empty_relat = Relat(title=relat_title)

    def test_story_should_have_a_title(self):
        document = """
        otro:otro
        """
        self.assertRaises(TypeError, Relat.create_from_document, document)

    def test_screens_are_created_correctly(self):

        # Screens are created correctly
        self.assertEqual(len(self.story_from_document.screens), 2)

    def test_options_are_created_correctly(self):
        self.assertEqual(len(self.story_from_document.options), 2)

    def test_cant_create_relat_with_invalidid_options(self):
        failed_document = """
        no: tienen
        sentido: estos
        cosos: no?
        """

        self.assertRaises(
            TypeError, Relat.create_from_document, failed_document)

    def test_relat_has_correct_number_of_endings(self):
        expected_endings = 4
        actual_endings = self.relat_from_document.get_number_endings

        self.assertEqual(expected_endings, actual_endings)

    def test_can_create_empty_relat(self):
        self.assertEqual(self.empty_relat.title, "A test Relat")

    def test_a_screens_can_be_added_to_story(self):
        screens = [
            "Screen 1",
            "Screen 2"
        ]

        for screen in screens:
            self.empty_relat.add_screen(screen)

        self.assertEqual(len(self.empty_relat.screens), 2)

        for actual, expected in zip(screens, self.empty_relat.screens):
            self.assertEqual(actual, expected)

    def test_an_option_can_be_added_to_relat(self):
        self.empty_relat.add_screen("A screen")

        self.empty_relat.add_option(
            Story(
                title="Option 1"
            )
        )

        # Test the option was added
        self.assertEqual(
            len(self.empty_relat.options),
            1
        )

        # Test the added option has the given title
        self.assertEqual(
            self.empty_relat.options[0].title,
            "Option 1"
        )

    def test_add_screen_to_story(self):

        empty_story = Story(title="A story", screens=[])

        screen = "A screen"

        empty_story.add_screen(screen)

        # Test screen was added
        self.assertEqual(
            len(empty_story.screens),
            1
        )

        # Test added screen is given screen
        self.assertEqual(
            empty_story.screens[0],
            screen
        )

    def test_an_option_can_be_added_to_story(self):
        empty_story = Story(title="A story", screens=[])

        screen = "A screen"

        empty_story.add_screen(screen)

        option = Story(title="Option 1", screens=["An option screen"])

        empty_story.add_option(option)

        # Test screen was added
        self.assertEqual(
            len(empty_story.options),
            1
        )

        # Test added screen is given screen
        self.assertEqual(
            empty_story.options[0].title,
            "Option 1"
        )

    def test_an_option_can_be_retrieved_navigating_story(self):

        option1 = self.story_from_document.get_option(0)

        self.assertEqual(
            option1.title,
            "option 1"
        )

        option2 = self.story_from_document.get_option(1)

        self.assertEqual(
            option2.title,
            "Option 2"
        )

        option121 = self.story_from_document\
            .get_option(0).get_option(1).get_option(0)

        self.assertEqual(
            option121.title,
            "option 1.2.1"
        )

    def test_shallow_equals_returns_true_if_title_and_screens_are_equal(self):
        o1 = Story(title="Option 1", screens=["O1 screen"])

        s1 = Story(
            title="Story 1",
            screens=["Screen 1"],
        )
        s1.options = [o1]

        s2 = Story(
            title="Story 1",
            screens=["Screen 1"],
            options=[]
        )

        s3 = Story(
            title="Story 1",
            screens=["Screen"],
            options=[]
        )

        self.assertTrue(
            s1.shallow_equal(s2)
        )

        self.assertFalse(
            s1.shallow_equal(s3)
        )

    def test_equals_returns_true_if_storys_are_similar_and_shallow_equal(self):
        o1 = Story(title="Option 1", screens=["O1 screen"])

        s1 = Story(
            title="Story 1",
            screens=["Screen 1"],
        )
        s1.options = [o1]

        s2 = Story(
            title="Story 1",
            screens=["Screen 1"],
        )
        s2.options = [o1]

        s3 = Story(
            title="Story 1",
            screens=["Screen"],
            options=[]
        )

        s4 = Story(
            title="Story 1",
            screens=["Screen 1"],
        )

        self.assertNotEqual(
            s1, 1
        )

        self.assertNotEqual(
            s1, s3
        )

        self.assertNotEqual(
            s1, s4
        )

        self.assertEqual(
            s1, s2
        )

    def test_navigating_with_valid_paths_works(self):
        path1 = [0, 0]

        self.assertEqual(self.relat_from_document.navigate(
            path1).screens[0], "option 1.1 screen 1")



from .story import Story

from typing import List


class Relat():
    """ Represents the start of an interactive story.

    Relat comes from the catalan word for _tale_

    Attributes:
        story (Story): a story, the entry point of the narrative
    """
    story: Story

    def __init__(self, title, *args, **kwargs):
        self.story = Story(title)

    @classmethod
    def create_from_document(cls, document):
        """Create a Relat from a document

        Arguments:
            document (dict): dict used for creating the document

        Returns:
            a Relat instance

        """
        story = Story(**document)

        instance = cls(story.title)
        instance.story = story

        return instance

    @property
    def get_number_endings(self):
        """Obtain the number of endings of the story

        Since any story in Relaty is represented in the shape
        of a tree, and ending is just a leaf.

        Returns:
            number of endings in the story
        """
        return self.story.get_number_endings

    @property
    def title(self):
        """Obtain the title of the Relat"""
        return self.story.title

    @property
    def screens(self):
        """Access the initial screens of the Relat"""
        return self.story.screens

    @property
    def options(self):
        """Access the root-level options of the Relat"""
        return self.story.options

    def add_screen(self, screen: str):
        """Adds a screen at the end of the initial screens"""
        self.story.add_screen(screen)

    def add_option(self, option: Story):
        """Adds and option to the root-level options of the Relat"""
        self.story.add_option(option)

    def navigate(self, path):
        """Returns the story at the path"""

        return self.story.navigate(path)

    def play(self):
        """Play the story in cli mode

        When this method is invoked, the story can be played in the terminal.
        It shows every screen and ask for confirmation to continue.

        Then displays each options and asks the user to choose one

        """
        self.story.play()



import typing


class Story():
    title: str
    screens: typing.List[str]
    options: typing.Union[typing.List['Story'], None]

    def __init__(
            self,
            title: str,
            screens: typing.List[str] = [],
            options: typing.Union[typing.List[typing.Dict], None] = None,
    ):

        self.title = title
        self.screens = screens

        if options is not None:
            self.options = [Story(**option) for option in options]
        else:
            self.options = options

    @property
    def get_number_endings(self):
        """Get the number of endings of the Story
        """
        if self.options is None:
            return 1
        else:
            return sum([option.get_number_endings for option in self.options])

    def display_options(self):
        if self.options is None:
            return

        for number, option in enumerate(self.options):
            print(f'{number+1} - {option.title}')

        try:
            option_number = int(input("Input option number: "))

            if option_number < 1 or option_number > len(self.options):
                raise ValueError

            return option_number
        except ValueError:
            print("The input wasn't valid, try again")
            self.display_options()

    def add_screen(self, screen: str):
        self.screens.append(screen)

    def add_option(self, option: 'Story'):
        if self.options is None:
            self.options = []
        self.options.append(option)

    def get_option(self, number):
        if self.options is None:
            return None
        return self.options[number]

    def shallow_equal(self, story: 'Story'):
        return \
            self.title == story.title and all(
                [a == b
                 for a, b in zip(self.screens, story.screens)])

    def __eq__(self, other):
        # If it's not a Story
        if not isinstance(other, Story):
            return False

        # If it's not shallow equal
        if not self.shallow_equal(other):
            return False

        if self.options is None or other.options is None:
            return False

        if len(self.options) != len(other.options):
            return False

        return self.options == other.options

    def navigate(self, path: typing.List[int]):
        result = self

        for option in path:
            result = result.get_option(option)

        return result

    def play(self):
        # Print title
        print(self.title)

        # Print each screen
        for s in self.screens:
            print(s)
            input("Print any key to continue...")

        option_number = self.display_options()

        if option_number is None:
            return

        self.options[option_number-1].play()



from distutils.core import setup
setup(
    name='relaty',         # How you named your package folder (MyLib)
    packages=['relaty'],   # Chose the same as "name"
    version='0.0.2',      # Start with a small number and increase it with every change you make
    # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    license='MIT',
    # Give a short description about your library
    description='Small package to build Choose Your Own Adventure-like stories',
    author='Pablo Toledo Margalef',                   # Type in your name
    author_email='pabloatm980@gmail.com',      # Type in your E-Mail
    # Provide either the link to your github or to your website
    url='https://github.com/papablo/relaty',
    # I explain this later on
    download_url='https://github.com/PaPablo/relaty/archive/v0.0.2.tar.gz',
    # Keywords that define your package best
    keywords=['fiction', 'interactive', 'writing'],
    install_requires=[            # I get to this in a second
        'click',
        'PyYAML',
    ],
    classifiers=[
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Development Status :: 3 - Alpha',
        # Define that your audience are developers
        'Intended Audience :: Developers',
        'Topic :: Games/Entertainment',
        'License :: OSI Approved :: MIT License',   # Again, pick a license
        # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3',
    ],
)
